@@ -6,11 +6,7 @@
 """Parse SQL statements."""
 
 
-__version__ = '0.1.4'
-
-
-class SQLParseError(Exception):
-    """Base class for exceptions in this module."""
+__version__ = '0.1.5'
 
 
 # Setup namespace
@@ -18,6 +14,9 @@ from sqlparse import engine
 from sqlparse import filters
 from sqlparse import formatter
 
+# Deprecated in 0.1.5. Will be removed in 0.2.0
+from sqlparse.exceptions import SQLParseError
+
 
 def parse(sql):
     """Parse sql and return a list of statements.
@@ -26,9 +25,17 @@ def parse(sql):
 
     Returns a tuple of :class:`~sqlparse.sql.Statement` instances.
     """
+    return tuple(parsestream(sql))
+
+
+def parsestream(stream):
+    """Parses sql statements from file-like object.
+
+    Returns a generator of Statement instances.
+    """
     stack = engine.FilterStack()
     stack.full_analyze()
-    return tuple(stack.run(sql))
+    return stack.run(stream)
 
 
 def format(sql, **options):
@@ -56,6 +63,8 @@ def split(sql):
 
 
 from sqlparse.engine.filter import StatementFilter
+
+
 def split2(stream):
     splitter = StatementFilter()
     return list(splitter.process(None, stream))
