@@ -55,7 +55,8 @@ def _group_matching(tlist, start_ttype, start_value, end_ttype, end_value,
                     cls, include_semicolon=False, recurse=False):
     def _find_matching(i, tl, stt, sva, ett, eva):
         depth = 1
-        for t in tl.tokens[i:]:
+        for n in xrange(i, len(tl.tokens)):
+            t = tl.tokens[n]
             if t.match(stt, sva):
                 depth += 1
             elif t.match(ett, eva):
@@ -137,10 +138,15 @@ def group_identifier(tlist):
                        or y.ttype is T.Operator
                        or y.ttype is T.Wildcard),
             lambda y: (y.ttype in (T.String.Symbol,
+                                   T.String.Single,
                                    T.Name,
                                    T.Wildcard,
                                    T.Literal.Number.Integer))))
         for t in tl.tokens[i:]:
+            # Don't take whitespaces into account.
+            if t.ttype is T.Whitespace:
+                yield t
+                continue
             if next(x)(t):
                 yield t
             else:
@@ -149,7 +155,8 @@ def group_identifier(tlist):
     def _next_token(tl, i):
         # chooses the next token. if two tokens are found then the
         # first is returned.
-        t1 = tl.token_next_by_type(i, (T.String.Symbol, T.Name))
+        t1 = tl.token_next_by_type(
+            i, (T.String.Symbol, T.String.Single, T.Name))
         t2 = tl.token_next_by_instance(i, sql.Function)
         if t1 and t2:
             i1 = tl.token_index(t1)
@@ -174,6 +181,9 @@ def group_identifier(tlist):
         identifier_tokens = [token] + list(
             _consume_cycle(tlist,
                            tlist.token_index(token) + 1))
+        # remove trailing whitespace
+        if identifier_tokens and identifier_tokens[-1].ttype is T.Whitespace:
+            identifier_tokens = identifier_tokens[:-1]
         if not (len(identifier_tokens) == 1
                 and isinstance(identifier_tokens[0], sql.Function)):
             group = tlist.group_tokens(sql.Identifier, identifier_tokens)
@@ -288,10 +298,11 @@ def group_aliased(tlist):
     while token:
         next_ = tlist.token_next(tlist.token_index(token))
         if next_ is not None and isinstance(next_, clss):
-            grp = tlist.tokens_between(token, next_)[1:]
-            token.tokens.extend(grp)
-            for t in grp:
-                tlist.tokens.remove(t)
+            if not next_.value.upper().startswith('VARCHAR'):
+                grp = tlist.tokens_between(token, next_)[1:]
+                token.tokens.extend(grp)
+                for t in grp:
+                    tlist.tokens.remove(t)
         idx = tlist.token_index(token) + 1
         token = tlist.token_next_by_instance(idx, clss)
 
