@@ -4,29 +4,19 @@ import re
 
 from os.path import abspath, join
 
-from sqlparse import sql
-from sqlparse import tokens as T
+from sqlparse import sql, tokens as T
 from sqlparse.engine import FilterStack
+from sqlparse.lexer import tokenize
+from sqlparse.pipeline import Pipeline
 from sqlparse.tokens import (Comment, Comparison, Keyword, Name, Punctuation,
                              String, Whitespace)
-
-
-class Filter(object):
-
-    def process(self, *args):
-        raise NotImplementedError
-
-
-class TokenFilter(Filter):
-
-    def process(self, stack, stream):
-        raise NotImplementedError
+from sqlparse.utils import memoize_generator
 
 
 # --------------------------
 # token process
 
-class _CaseFilter(TokenFilter):
+class _CaseFilter:
 
     ttype = None
 
@@ -57,7 +47,7 @@ class IdentifierCaseFilter(_CaseFilter):
             yield ttype, value
 
 
-class GetComments(Filter):
+class GetComments:
     """Get the comments from a stack"""
     def process(self, stack, stream):
         for token_type, value in stream:
@@ -65,7 +55,7 @@ class GetComments(Filter):
                 yield token_type, value
 
 
-class StripComments(Filter):
+class StripComments:
     """Strip the comments from a stack"""
     def process(self, stack, stream):
         for token_type, value in stream:
@@ -101,15 +91,20 @@ def StripWhitespace(stream):
         last_type = token_type
 
 
-class IncludeStatement(Filter):
+class IncludeStatement:
     """Filter that enable a INCLUDE statement"""
 
-    def __init__(self, dirpath=".", maxRecursive=10):
+    def __init__(self, dirpath=".", maxrecursive=10, raiseexceptions=False):
+        if maxrecursive <= 0:
+            raise ValueError('Max recursion limit reached')
+
         self.dirpath = abspath(dirpath)
-        self.maxRecursive = maxRecursive
+        self.maxRecursive = maxrecursive
+        self.raiseexceptions = raiseexceptions
 
         self.detected = False
 
+    @memoize_generator
     def process(self, stack, stream):
         # Run over all tokens in the stream
         for token_type, value in stream:
@@ -122,30 +117,48 @@ class IncludeStatement(Filter):
             elif self.detected:
                 # Omit whitespaces
                 if token_type in Whitespace:
-                    pass
-
-                # Get path of file to include
-                path = None
+                    continue
 
+                # Found file path to include
                 if token_type in String.Symbol:
 #                if token_type in tokens.String.Symbol:
+
+                    # Get path of file to include
                     path = join(self.dirpath, value[1:-1])
 
-                # Include file if path was found
-                if path:
                     try:
                         f = open(path)
                         raw_sql = f.read()
                         f.close()
+
+                    # There was a problem loading the include file
                     except IOError, err:
+                        # Raise the exception to the interpreter
+                        if self.raiseexceptions:
+                            raise
+
+                        # Put the exception as a comment on the SQL code
                         yield Comment, u'-- IOError: %s\n' % err
 
                     else:
                         # Create new FilterStack to parse readed file
                         # and add all its tokens to the main stack recursively
-                        # [ToDo] Add maximum recursive iteration value
+                        try:
+                            filtr = IncludeStatement(self.dirpath,
+                                                     self.maxRecursive - 1,
+                                                     self.raiseexceptions)
+
+                        # Max recursion limit reached
+                        except ValueError, err:
+                            # Raise the exception to the interpreter
+                            if self.raiseexceptions:
+                                raise
+
+                            # Put the exception as a comment on the SQL code
+                            yield Comment, u'-- ValueError: %s\n' % err
+
                         stack = FilterStack()
-                        stack.preprocess.append(IncludeStatement(self.dirpath))
+                        stack.preprocess.append(filtr)
 
                         for tv in stack.run(raw_sql):
                             yield tv
@@ -163,7 +176,7 @@ class IncludeStatement(Filter):
 # ----------------------
 # statement process
 
-class StripCommentsFilter(Filter):
+class StripCommentsFilter:
 
     def _get_next_comment(self, tlist):
         # TODO(andi) Comment types should be unified, see related issue38
@@ -194,7 +207,7 @@ class StripCommentsFilter(Filter):
         self._process(stmt)
 
 
-class StripWhitespaceFilter(Filter):
+class StripWhitespaceFilter:
 
     def _stripws(self, tlist):
         func_name = '_stripws_%s' % tlist.__class__.__name__.lower()
@@ -219,14 +232,14 @@ class StripWhitespaceFilter(Filter):
         self._stripws_default(tlist)
 
     def process(self, stack, stmt, depth=0):
-        [self.process(stack, sgroup, depth+1)
+        [self.process(stack, sgroup, depth + 1)
          for sgroup in stmt.get_sublists()]
         self._stripws(stmt)
         if depth == 0 and stmt.tokens[-1].is_whitespace():
             stmt.tokens.pop(-1)
 
 
-class ReindentFilter(Filter):
+class ReindentFilter:
 
     def __init__(self, width=2, char=' ', line_width=None):
         self.width = width
@@ -316,8 +329,8 @@ class ReindentFilter(Filter):
             self.indent += 1
             tlist.tokens.insert(0, self.nl())
             indented = True
-        num_offset = self._get_offset(tlist.token_next_match(0,
-                                                        T.Punctuation, '('))
+        num_offset = self._get_offset(
+            tlist.token_next_match(0, T.Punctuation, '('))
         self.offset += num_offset
         self._process_default(tlist, stmts=not indented)
         if indented:
@@ -384,17 +397,17 @@ class ReindentFilter(Filter):
                     nl = '\n'
                 else:
                     nl = '\n\n'
-                stmt.tokens.insert(0,
-                    sql.Token(T.Whitespace, nl))
+                stmt.tokens.insert(
+                    0, sql.Token(T.Whitespace, nl))
             if self._last_stmt != stmt:
                 self._last_stmt = stmt
 
 
 # FIXME: Doesn't work ;)
-class RightMarginFilter(Filter):
+class RightMarginFilter:
 
     keep_together = (
-#        sql.TypeCast, sql.Identifier, sql.Alias,
+        # sql.TypeCast, sql.Identifier, sql.Alias,
     )
 
     def __init__(self, width=79):
@@ -429,7 +442,7 @@ class RightMarginFilter(Filter):
         group.tokens = self._process(stack, group, group.tokens)
 
 
-class ColumnsSelect(Filter):
+class ColumnsSelect:
     """Get the columns names of a SELECT query"""
     def process(self, stack, stream):
         mode = 0
@@ -483,7 +496,7 @@ class ColumnsSelect(Filter):
 # ---------------------------
 # postprocess
 
-class SerializerUnicode(Filter):
+class SerializerUnicode:
 
     def process(self, stack, stmt):
         raw = unicode(stmt)
@@ -493,6 +506,7 @@ class SerializerUnicode(Filter):
             res += '\n'
         return res
 
+
 def Tokens2Unicode(stream):
     result = ""
 
@@ -502,14 +516,32 @@ def Tokens2Unicode(stream):
     return result
 
 
-class OutputPythonFilter(Filter):
+class OutputFilter:
+    varname_prefix = ''
 
     def __init__(self, varname='sql'):
-        self.varname = varname
-        self.cnt = 0
+        self.varname = self.varname_prefix + varname
+        self.count = 0
 
-    def _process(self, stream, varname, count, has_nl):
-        if count > 1:
+    def _process(self, stream, varname, has_nl):
+        raise NotImplementedError
+
+    def process(self, stack, stmt):
+        self.count += 1
+        if self.count > 1:
+            varname = '%s%d' % (self.varname, self.count)
+        else:
+            varname = self.varname
+
+        has_nl = len(unicode(stmt).strip().splitlines()) > 1
+        stmt.tokens = self._process(stmt.tokens, varname, has_nl)
+        return stmt
+
+
+class OutputPythonFilter(OutputFilter):
+    def _process(self, stream, varname, has_nl):
+        # SQL query asignation to varname
+        if self.count > 1:
             yield sql.Token(T.Whitespace, '\n')
         yield sql.Token(T.Name, varname)
         yield sql.Token(T.Whitespace, ' ')
@@ -518,85 +550,87 @@ class OutputPythonFilter(Filter):
         if has_nl:
             yield sql.Token(T.Operator, '(')
         yield sql.Token(T.Text, "'")
-        cnt = 0
+
+        # Print the tokens on the quote
         for token in stream:
-            cnt += 1
+            # Token is a new line separator
             if token.is_whitespace() and '\n' in token.value:
-                if cnt == 1:
-                    continue
-                after_lb = token.value.split('\n', 1)[1]
+                # Close quote and add a new line
                 yield sql.Token(T.Text, " '")
                 yield sql.Token(T.Whitespace, '\n')
-                for i in range(len(varname) + 4):
-                    yield sql.Token(T.Whitespace, ' ')
+
+                # Quote header on secondary lines
+                yield sql.Token(T.Whitespace, ' ' * (len(varname) + 4))
                 yield sql.Token(T.Text, "'")
-                if after_lb:  # it's the indendation
+
+                # Indentation
+                after_lb = token.value.split('\n', 1)[1]
+                if after_lb:
                     yield sql.Token(T.Whitespace, after_lb)
                 continue
-            elif token.value and "'" in token.value:
+
+            # Token has escape chars
+            elif "'" in token.value:
                 token.value = token.value.replace("'", "\\'")
-            yield sql.Token(T.Text, token.value or '')
+
+            # Put the token
+            yield sql.Token(T.Text, token.value)
+
+        # Close quote
         yield sql.Token(T.Text, "'")
         if has_nl:
             yield sql.Token(T.Operator, ')')
 
-    def process(self, stack, stmt):
-        self.cnt += 1
-        if self.cnt > 1:
-            varname = '%s%d' % (self.varname, self.cnt)
-        else:
-            varname = self.varname
-        has_nl = len(unicode(stmt).strip().splitlines()) > 1
-        stmt.tokens = self._process(stmt.tokens, varname, self.cnt, has_nl)
-        return stmt
 
+class OutputPHPFilter(OutputFilter):
+    varname_prefix = '$'
 
-class OutputPHPFilter(Filter):
-
-    def __init__(self, varname='sql'):
-        self.varname = '$%s' % varname
-        self.count = 0
-
-    def _process(self, stream, varname):
+    def _process(self, stream, varname, has_nl):
+        # SQL query asignation to varname (quote header)
         if self.count > 1:
             yield sql.Token(T.Whitespace, '\n')
         yield sql.Token(T.Name, varname)
         yield sql.Token(T.Whitespace, ' ')
+        if has_nl:
+            yield sql.Token(T.Whitespace, ' ')
         yield sql.Token(T.Operator, '=')
         yield sql.Token(T.Whitespace, ' ')
         yield sql.Token(T.Text, '"')
+
+        # Print the tokens on the quote
         for token in stream:
+            # Token is a new line separator
             if token.is_whitespace() and '\n' in token.value:
-                after_lb = token.value.split('\n', 1)[1]
-                yield sql.Token(T.Text, ' "')
-                yield sql.Token(T.Operator, ';')
+                # Close quote and add a new line
+                yield sql.Token(T.Text, ' ";')
                 yield sql.Token(T.Whitespace, '\n')
+
+                # Quote header on secondary lines
                 yield sql.Token(T.Name, varname)
                 yield sql.Token(T.Whitespace, ' ')
-                yield sql.Token(T.Punctuation, '.')
-                yield sql.Token(T.Operator, '=')
+                yield sql.Token(T.Operator, '.=')
                 yield sql.Token(T.Whitespace, ' ')
                 yield sql.Token(T.Text, '"')
+
+                # Indentation
+                after_lb = token.value.split('\n', 1)[1]
                 if after_lb:
-                    yield sql.Token(T.Text, after_lb)
+                    yield sql.Token(T.Whitespace, after_lb)
                 continue
+
+            # Token has escape chars
             elif '"' in token.value:
                 token.value = token.value.replace('"', '\\"')
+
+            # Put the token
             yield sql.Token(T.Text, token.value)
+
+        # Close quote
         yield sql.Token(T.Text, '"')
         yield sql.Token(T.Punctuation, ';')
 
-    def process(self, stack, stmt):
-        self.count += 1
-        if self.count > 1:
-            varname = '%s%d' % (self.varname, self.count)
-        else:
-            varname = self.varname
-        stmt.tokens = tuple(self._process(stmt.tokens, varname))
-        return stmt
 
-
-class Limit(Filter):
+class Limit:
     """Get the LIMIT of a query.
 
     If not defined, return -1 (SQL specification for no LIMIT query)
@@ -615,3 +649,13 @@ class Limit(Filter):
                 return stream[4 - index][1]
 
         return -1
+
+
+def compact(stream):
+    """Function that return a compacted version of the stream"""
+    pipe = Pipeline()
+
+    pipe.append(StripComments())
+    pipe.append(StripWhitespace)
+
+    return pipe(stream)
