@@ -15,18 +15,20 @@ class Token(object):
     the type of the token.
     """
 
-    __slots__ = ('value', 'ttype', 'parent')
+    __slots__ = ('value', 'ttype', 'parent', 'normalized', 'is_keyword')
 
     def __init__(self, ttype, value):
         self.value = value
+        self.normalized = value.upper() if ttype in T.Keyword else value
         self.ttype = ttype
+        self.is_keyword = ttype in T.Keyword
         self.parent = None
 
     def __str__(self):
         return unicode(self).encode('utf-8')
 
     def __repr__(self):
-        short = self._get_repr_value()
+        short = self._get_repr_value().encode('utf-8')
         return '<%s \'%s\' at 0x%07x>' % (self._get_repr_name(),
                                           short, id(self))
 
@@ -36,7 +38,7 @@ class Token(object):
 
     def to_unicode(self):
         """Returns a unicode representation of this object.
-        
+
         @deprecated: please use __unicode__()
         """
         return unicode(self)
@@ -47,10 +49,8 @@ class Token(object):
     def _get_repr_value(self):
         raw = unicode(self)
         if len(raw) > 7:
-            short = raw[:6] + u'...'
-        else:
-            short = raw
-        return re.sub('\s+', ' ', short)
+            raw = raw[:6] + u'...'
+        return re.sub('\s+', ' ', raw)
 
     def flatten(self):
         """Resolve subgroups."""
@@ -71,23 +71,33 @@ class Token(object):
         type_matched = self.ttype is ttype
         if not type_matched or values is None:
             return type_matched
-        if isinstance(values, basestring):
-            values = set([values])
+
         if regex:
+            if isinstance(values, basestring):
+                values = set([values])
+
             if self.ttype is T.Keyword:
-                values = set([re.compile(v, re.IGNORECASE) for v in values])
+                values = set(re.compile(v, re.IGNORECASE) for v in values)
             else:
-                values = set([re.compile(v) for v in values])
+                values = set(re.compile(v) for v in values)
+
             for pattern in values:
                 if pattern.search(self.value):
                     return True
             return False
-        else:
-            if self.ttype in T.Keyword:
-                values = set([v.upper() for v in values])
-                return self.value.upper() in values
-            else:
-                return self.value in values
+
+        if isinstance(values, basestring):
+            if self.is_keyword:
+                return values.upper() == self.normalized
+            return values == self.value
+
+        if self.is_keyword:
+            for v in values:
+                if v.upper() == self.normalized:
+                    return True
+            return False
+
+        return self.value in values
 
     def is_group(self):
         """Returns ``True`` if this object has children."""
@@ -159,6 +169,14 @@ class TokenList(Token):
             if (token.is_group() and (max_depth is None or depth < max_depth)):
                 token._pprint_tree(max_depth, depth + 1)
 
+    def _remove_quotes(self, val):
+        """Helper that removes surrounding quotes from strings."""
+        if not val:
+            return val
+        if val[0] in ('"', '\'') and val[-1] == val[0]:
+            val = val[1:-1]
+        return val
+
     def flatten(self):
         """Generator yielding ungrouped tokens.
 
@@ -171,6 +189,13 @@ class TokenList(Token):
             else:
                 yield token
 
+#    def __iter__(self):
+#        return self
+#
+#    def next(self):
+#        for token in self.tokens:
+#            yield token
+
     def is_group(self):
         return True
 
@@ -203,12 +228,9 @@ class TokenList(Token):
 
         If no matching token can be found ``None`` is returned.
         """
-        if isinstance(clss, (list, tuple)):
+        if not isinstance(clss, (list, tuple)):
             clss = (clss,)
 
-        if isinstance(clss, tuple):
-            clss = tuple(clss)
-
         for token in self.tokens[idx:]:
             if isinstance(token, clss):
                 return token
@@ -227,7 +249,8 @@ class TokenList(Token):
         if not isinstance(idx, int):
             idx = self.token_index(idx)
 
-        for token in self.tokens[idx:]:
+        for n in xrange(idx, len(self.tokens)):
+            token = self.tokens[n]
             if token.match(ttype, value, regex):
                 return token
 
@@ -244,7 +267,7 @@ class TokenList(Token):
 
     def token_matching(self, idx, funcs):
         for token in self.tokens[idx:]:
-            for i, func in enumerate(funcs):
+            for func in funcs:
                 if func(token):
                     return token
 
@@ -260,7 +283,7 @@ class TokenList(Token):
         if not isinstance(idx, int):
             idx = self.token_index(idx)
 
-        while idx != 0:
+        while idx:
             idx -= 1
             if self.tokens[idx].is_whitespace() and skip_ws:
                 continue
@@ -344,11 +367,13 @@ class TokenList(Token):
         else:
             next_ = self.token_next_by_instance(0, Identifier)
             if next_ is None:
-                return None
+                next_ = self.token_next_by_type(0, T.String.Symbol)
+                if next_ is None:
+                    return None
             alias = next_
         if isinstance(alias, Identifier):
             return alias.get_name()
-        return unicode(alias)
+        return self._remove_quotes(unicode(alias))
 
     def get_name(self):
         """Returns the name of this identifier.
@@ -367,13 +392,16 @@ class TokenList(Token):
         # a.b
         dot = self.token_next_match(0, T.Punctuation, '.')
         if dot is None:
-            return self.token_next_by_type(0, T.Name).value
-        else:
-            next_ = self.token_next_by_type(self.token_index(dot),
-                                            (T.Name, T.Wildcard))
-            if next_ is None:  # invalid identifier, e.g. "a."
-                return None
-            return next_.value
+            next_ = self.token_next_by_type(0, T.Name)
+            if next_ is not None:
+                return self._remove_quotes(next_.value)
+            return None
+
+        next_ = self.token_next_by_type(self.token_index(dot),
+                                        (T.Name, T.Wildcard, T.String.Symbol))
+        if next_ is None:  # invalid identifier, e.g. "a."
+            return None
+        return self._remove_quotes(next_.value)
 
 
 class Statement(TokenList):
@@ -395,7 +423,7 @@ class Statement(TokenList):
             return 'UNKNOWN'
 
         elif first_token.ttype in (T.Keyword.DML, T.Keyword.DDL):
-            return first_token.value.upper()
+            return first_token.normalized
 
         return 'UNKNOWN'
 
@@ -419,7 +447,7 @@ class Identifier(TokenList):
         prev_ = self.token_prev(self.token_index(dot))
         if prev_ is None:  # something must be verry wrong here..
             return None
-        return prev_.value
+        return self._remove_quotes(prev_.value)
 
     def is_wildcard(self):
         """Return ``True`` if this identifier contains a wildcard."""
