@@ -53,6 +53,12 @@ class TestGrouping(TestCaseBase):
         self.ndiffAssertEqual(s, unicode(parsed))
         self.assert_(isinstance(parsed.tokens[-1].tokens[3], sql.Identifier))
 
+        s = "INSERT INTO `test` VALUES('foo', 'bar');"
+        parsed = sqlparse.parse(s)[0]
+        types = [l.ttype for l in parsed.tokens if not l.is_whitespace()]
+        self.assertEquals(types, [T.DML, T.Keyword, None,
+                                  T.Keyword, None, T.Punctuation])
+
     def test_identifier_wildcard(self):
         p = sqlparse.parse('a.*, b.id')[0]
         self.assert_(isinstance(p.tokens[0], sql.IdentifierList))
@@ -183,6 +189,10 @@ class TestGrouping(TestCaseBase):
         self.assert_(isinstance(p.tokens[0], sql.Function))
         self.assertEqual(len(list(p.tokens[0].get_parameters())), 2)
 
+    def test_varchar(self):
+        p = sqlparse.parse('"text" Varchar(50) NOT NULL')[0]
+        self.assert_(isinstance(p.tokens[2], sql.Function))
+
 
 class TestStatement(TestCaseBase):
 
@@ -197,3 +207,27 @@ class TestStatement(TestCaseBase):
         # are parsed as two statements where later only consists of the
         # trailing whitespace.
         self.assertEqual(f('\n').get_type(), 'UNKNOWN')
+
+
+def test_identifier_with_operators():  # issue 53
+    p = sqlparse.parse('foo||bar')[0]
+    assert len(p.tokens) == 1
+    assert isinstance(p.tokens[0], sql.Identifier)
+    # again with whitespaces
+    p = sqlparse.parse('foo || bar')[0]
+    assert len(p.tokens) == 1
+    assert isinstance(p.tokens[0], sql.Identifier)
+
+
+def test_identifier_with_op_trailing_ws():
+    # make sure trailing whitespace isn't grouped with identifier
+    p = sqlparse.parse('foo || bar ')[0]
+    assert len(p.tokens) == 2
+    assert isinstance(p.tokens[0], sql.Identifier)
+    assert p.tokens[1].ttype is T.Whitespace
+
+
+def test_identifier_string_concat():
+    p = sqlparse.parse('\'foo\' || bar')[0]
+    assert len(p.tokens) == 1
+    assert isinstance(p.tokens[0], sql.Identifier)
