@@ -20,7 +20,7 @@ class SQLParseTest(TestCaseBase):
     def test_multistatement(self):
         sql1 = 'select * from foo;'
         sql2 = 'select * from bar;'
-        stmts = sqlparse.parse(sql1+sql2)
+        stmts = sqlparse.parse(sql1 + sql2)
         self.assertEqual(len(stmts), 2)
         self.assertEqual(str(stmts[0]), sql1)
         self.assertEqual(str(stmts[1]), sql2)
@@ -106,3 +106,10 @@ class SQLParseTest(TestCaseBase):
         t = sqlparse.parse('foo.key')[0].tokens
         self.assertEqual(len(t), 1)
         self.assert_(isinstance(t[0], sqlparse.sql.Identifier))
+
+
+def test_quoted_identifier():
+    t = sqlparse.parse('select x.y as "z" from foo')[0].tokens
+    assert isinstance(t[2], sqlparse.sql.Identifier)
+    assert t[2].get_name() == 'z'
+    assert t[2].get_real_name() == 'y'
